##############################################################################
#
#	helper.pm	-	Helper class
#	
#	This file is part of the COST-ET tool written by 
#	Marios Kleanthous, Damien Hardy, Isidoros Sideris and Yiannakis Sazeides
#	University of Cyprus
#	The research leading to this tool is supported by the European Commission
#	FP7 project ”Energy-conscious 3D Server-on-Chip for Green Cloud
#	Services (Project No:247779 ”EuroCloud”)”
#
#	This source file is distributed "as is" in the hope that it will be useful.
#	The tool comes with no warranty, and no author or distributor accepts any
#	responsability for the consequences of its use.
#
#	Everyone is granted permission to copy, modify and redistribute
#	this tool set under the following conditions:
#
#		This source code is distributed for non-commercial use only.
#		Please contact the maintainer for restrictions applying to
#		commercial use.
#
#		Permission is granted to anyone to make or distribute copies
#		of this source code, either as received or modified, in any
#		medium, provided that all copyright notices, permission and
#		nonwarranty notices are preserved, and that the distributor
#		grants the recipient permission for further redistribution as
#		permitted by this document.
#
#		Permission is granted to distribute this file in compiled
#		or executable form under the same conditions that apply for
#		source code, provided that either:
#
#		A.	it is accompanied by the corresponding machine-readable
#			source code,
#		B.	it is accompanied by a written offer, with no time limit,
#			to give anyone a machine-readable copy of the corresponding
#			source code in return for reimbursement of the cost of
#			distribution.  This written offer must permit verbatim
#			duplication by anyone, or
#		C.	it is distributed by someone who received only the
#			executable form, and is accompanied by a copy of the
#			written offer of source code that they received concurrently.
#
#	In other words, you are welcome to use, share and improve this
#	source file.  You are forbidden to forbid anyone else to use, share
#	and improve what you give them.
#
#	The research leading to this tool is supported by the European Commission 
#	FP7 project "Energy-conscious 3D Server-on-Chip for Green Cloud Services 
#	(Project No:247779 "EuroCloud")".
#
#	Tool: http://www.cs.ucy.ac.cy/carch/xi/eetco.php
#	Project: http://www.eurocloudserver.com/
#	European Commission FP7: http://cordis.europa.eu/fp7/home_en.html
#
##############################################################################

package helper;

use strict;
use warnings;


################################################
#
#	constructor
#
################################################
sub new()
{
	my ($class,)= @_;
	my $self={};
	bless ($self , $class);
	return $self;
}


################################################
#
#	parserDCFile function
#
#	input:
#		file path: /user/r.params
#
#	result:
#		pointer to a hashmap of parameters
#
################################################
sub parserDCFile($)
{
	my ($class,$file)= @_;
	
	my %res;
	
	if(!open FILE,"<$file"){die "cannot open file: $file\n";}

	while(defined(my $line=<FILE>))
	{
		if($line =~ /^\s*(\w+)\s+(\d+(?:.\d+)?(?:e[+|-]?\d+)?)/)
		{
			$res{"$1"}=$2;
		}
	}
	close FILE;

	return \%res;
}


################################################
#
#	parserResourceFile function
#
#	input:
#		file path: /user/r.params
#
#	result:
#		pointer to a hashmap of parameters
#
################################################
sub parserResourceFile($)
{
	my ($class,$file)= @_;
	
	my %res;

	if(!open FILE,"<$file"){die "cannot open file: $file\n";}

	while(defined(my $line=<FILE>))
	{
		if($line =~ /^\s*(\w+)\s+(\d+(?:.\d+)?(?:e[+|-]?\d+)?)/)
		{
			$res{"$1"}=$2;
		}
		elsif($line =~ /^\s*resource_name\s+(\w+)/)
		{
			$res{resource_name}=$1;
		}
	}
	close FILE;
	
	return \%res;
}

###########################################
#
#	checkInputParameterName function
#
#	parameter: 
#		name: an input parameter name
#
#	result:
#		0 if the parameter is a DC parameter
#		1 if the parameter is a resource parameter
#		-1 if the parameter does not exist
#
#
###########################################
sub checkInputParameterName($)
{
	my ($class,$name)= @_;
	
	my %input =
	(
		"a_coolingarea", 0,
		"c_coolpowerdeplperW", 0,
		"p_othermodulesidle", 0,
		"c_nonsrvmodule", 0,
		"c_elecpeak", 0,
		"n_racks_othermodules", 0,
		"n_othermodulesrepl", 0,
		"salary_perrackpermonth", 0,
		"c_buildingpersqm", 0,
		"PUE", 0,
		"n_othermodules", 0,
		"report_resources", 0,
		"a_othermodules", 0,
		"c_electperKWH", 0,
		"dcdepr", 0,
		"u_othermodules", 0,
		"depr_othermodules", 0,
		"p_othermodules", 0,
		"cost_of_money", 0,
	
		#########
		"c_system", 1,
		"k_chipspersrvmodule", 1,
		"mttf_dram", 1,
		"p_dram_idle", 1,
		"p_system_idle", 1,
		"p_switchport", 1,
		"p_dram", 1,
		"mttf_coef", 1,
		"k_bladesperrack", 1,
		"p_switchport_idle", 1,
		"u", 1,
		"c_other", 1,
		"mttr", 1,
		"networkdepr", 1,
		"chip_perf", 1,
		"minimumdistance", 1,
		"SPUE", 1,
		"depth", 1,
		"pvf", 1,
		"c_switchport", 1,
		"srvdepr", 1,
		"resource_name", 1,
		"c_networkperrack", 1,
		"p_system", 1,
		"network_peak_powerperrack", 1,
		"width", 1,
		"coef_maintenance", 1,
		"mttf_switchport", 1,
		"p_chip", 1,
		"useddistance", 1,
		"p_other_idle", 1,
		"c_dram", 1,
		"k_modulesperblade", 1,
		"p_chip_idle", 1,
		"p_disk", 1,
		"mttf_disk", 1,
		"n_srvmodulesreq", 1,
		"c_disk", 1,
		"chip_mttf", 1,
		"p_other", 1,
		"height", 1,
		"n_disk", 1,
		"c_chip", 1,
		"n_dram", 1,
		"n_switchport", 1,
		"p_disk_idle", 1,
		"nb_time_step", 1,
	);
	
	if(!defined($input{$name}))
	{
		print("Warning: the input parameter '$name' does not exist\n");
		return -1;
	}
	else
	{
		return $input{$name};
	}
}


###########################################
#
#	checkOutputValueName function
#
#	parameter: 
#		name: an output value name
#
#	result:
#		0 if the output value name exist and is produce by the DC
#		1 if the output value name exist and is produce only by the Resource
#		-1 if the output value name doest notexist
#
#
###########################################
sub checkOutputValueName($)
{
	my ($class,$name)= @_;
	
	my %output =
	(
		#dc
		"tco",0,
		"c_infrastructure",0,
		"c_server",0,
		"c_power",0,
		"c_network",0,
		"c_maintenance",0,
		"n_racks",0,
		"a_dc",0,
		"p_total_avg",0,
		"p_total_peak",0,
		"co2peryr",0,
	
		#resource
		"a_perrack",1,
		"p_peakperrack",1,
		"n_srvmodules",1,
		"n_coldsp",1,
		"c_srvmodule",1,
		"availability",1,
		"Performance",1,
		"PerformanceOverTCO",1,
		"PerformanceOverWatt",1,
	);
	
	if(!defined($output{$name}))
	{
		print("Warning: the output value '$name' does not exist\n");
		return -1;
	}
	else
	{
		return $output{$name};
	}
}


###########################################
#
#	createParameterFile function
#
#	parameter: 
#		file path: /user/r.params
#		pointer to a hashmap of parameters
#
#	result:
#		the file is created with all the
#		parameters
#
#	remarks:
#		the file is deleted (if present)
#		before its creation
#
###########################################
sub createParameterFile($$)
{	
	my ($class,$file,$ptr)= @_;
	my %m= %{$ptr};
	
	#delete the file
	if (-e "$file"){unlink($file);}
	
	#create the resource configuration file
	open(FILE,">$file") || die ("Error when opening $file") ;
	while (my ($k,$v) = each(%m))
	{
		print (FILE "$k $v\n");
	}
	close FILE;
}


###########################################
#
#	callKernel function
#
#	parameter:
#		dc_configuration_file 
#		resource_configuration_file
#		ouput_file
#
#	result:
#		the ouput_file is created with the
#		outputs of the kernel
#
############################################
sub callKernel($$$)
{
	my ($class,$input_file_dc,$input_file_r,$output)= @_;
	
	my $basedir="../..";
	my $currentdir=`pwd`;
	chomp $currentdir;
	chdir($basedir) or die "$!";
	
	system("./cost-et.pl $input_file_dc $input_file_r  > $output");

	chdir($currentdir) or die "$!";
}


###########################################
#
#	parserResultKernel function
#
#	parameter:
#		kernel ouput file
#
#	result:
#		pointer to a hashmap of results
#
############################################
sub parserResultKernel($)
{
	my ($class,$file)= @_;

	my %res;
	
	if(!open FILE,"<$file"){die "cannot open file: $file\n";}
	
	while(defined(my $line=<FILE>))
	{
		if($line =~ /^\s*(\w+):\s+(\d+(?:.\d+)?(?:e[+|-]?\d+)?)/)
		{
			$res{"$1"}=$2;
		}
	}
	close FILE;
	
	return \%res;
	
}


###########################################
#
#	deleteFile function
#
#	parameter: 
#		file path: /user/r.params
#
#	result:
#		the file is deleted 
#
###########################################
sub deleteFile($)
{	
	my ($class,$file)= @_;
	
	#delete the  file
	if (-e "$file"){unlink($file);}
}

###########################################
#
#	deleteFileInDir function
#
#	parameter: 
#		dir path: /user/cost-et/wrappers/output
#
#	result:
#		All the file of dir are deleted 
#
###########################################
sub deleteFileInDir($)
{	
	my ($class,$dir)= @_;

	my $currentdir=`pwd`;
	chomp $currentdir;
	chdir($dir) or die "$!";
	
	foreach (<*>)
	{
		unlink($_) or die "Delete Failed: $!";
	}
	
	chdir($currentdir) or die "$!";
}


###########################################
#
#	makeGnuplotCurves2D function
#
#	parameter: 
#		$input_file (with absolute path): data for gnuplot
#		$output_file_path: the directory to put the output_file
#		$output_file_name to save the graph (without extension, the extension will be .eps)
#		$x_name: name of x-axis
#		$y_name: name of y-axis
#		$ptr_curves_names: pointer to a list containig the name of each curve to draw
#
#	result:
#		the graph is save in $output_file_path/$output_file_name.eps
#
###########################################
sub makeGnuplotCurves2D($$$$$)
{
	my ($class,$input_file,$output_file_path,$output_file_name,$x_name,$y_name,$ptr_curves_names)= @_;
	my @curves_names=@{$ptr_curves_names};
	my $nbcurves= $#curves_names+1;
	
	#remove previous output file
	if (-e "$output_file_path/$output_file_name".".eps"){unlink("$output_file_path/$output_file_name".".eps");}
	
	#remove template file if any
	if (-e "$output_file_path/template.plot"){unlink("$output_file_path/template.plot");}
	
	open(OUTPUTFILE,">$output_file_path/template.plot") || die ("Error when opening $output_file_path/template.plot") ;
	
	print(OUTPUTFILE "set xtics font \"Times-Roman,24\"\n");
	print(OUTPUTFILE "set ytics font \"Times-Roman,24\"\n");
	print(OUTPUTFILE "set xlabel font \"Times-Roman,24\"\n");
	print(OUTPUTFILE "set ylabel font \"Times-Roman,24\"\n");
	print(OUTPUTFILE "set xlabel \"$x_name\"\n");
	print(OUTPUTFILE "set ylabel \"$y_name\"\n");
	
	print(OUTPUTFILE "plot ");
	for(my $i=2;$i<=$nbcurves+1;$i++)
	{
		print(OUTPUTFILE "'$input_file' using 1:$i title '$curves_names[$i-2]' with linespoints");
		if($i<$nbcurves+1){print(OUTPUTFILE ",");}
	}
	print(OUTPUTFILE "\n");
	
	print(OUTPUTFILE "set term postscript color\n");
	print(OUTPUTFILE "set term postscript eps\n");
	print(OUTPUTFILE "set output '$output_file_path/$output_file_name.eps'\n");
	print(OUTPUTFILE "set size 1, 1\n");
	print(OUTPUTFILE "replot\n");
	
	close(OUTPUTFILE);

	system("gnuplot <$output_file_path/template.plot");

	#remove template file
	if (-e "$output_file_path/template.plot"){unlink("$output_file_path/template.plot");}
}


###########################################
#
#	makeGnuplotCurves3D function
#
#	parameter: 
#		$input_file (with absolute path): data for gnuplot
#		$output_file_path: the directory to put the output_file
#		$output_file_name to save the graph (without extension, the extension will be .eps)
#		$x_name: name of x-axis
#		$y_name: name of y-axis
#		$z_name: name of z-axis
#
#	result:
#		the graph is save in $output_file_path/$output_file_name.eps
#
###########################################
sub makeGnuplotCurves3D($$$$$)
{
	my ($class,$input_file,$output_file_path,$output_file_name,$x_name,$y_name,$z_name)= @_;
	
	#remove previous output file
	if (-e "$output_file_path/$output_file_name".".eps"){unlink("$output_file_path/$output_file_name".".eps");}
	
	#remove template file if any
	if (-e "$output_file_path/template.plot"){unlink("$output_file_path/template.plot");}
	
	open(OUTPUTFILE,">$output_file_path/template.plot") || die ("Error when opening $output_file_path/template.plot") ;
	
	
	print(OUTPUTFILE "set pm3d\n");
	
	print(OUTPUTFILE "set xtics font \"Times-Roman,24\"\n");
	print(OUTPUTFILE "set ytics font \"Times-Roman,24\"\n");
	print(OUTPUTFILE "set ztics font \"Times-Roman,24\"\n");
	print(OUTPUTFILE "set xlabel font \"Times-Roman,24\"\n");
	print(OUTPUTFILE "set ylabel font \"Times-Roman,24\"\n");
	print(OUTPUTFILE "set zlabel font \"Times-Roman,24\"\n");
	print(OUTPUTFILE "set xlabel \"$x_name\"\n");
	print(OUTPUTFILE "set ylabel \"$y_name\"\n");
	print(OUTPUTFILE "set zlabel \"$z_name\"\n");
	
	
	print(OUTPUTFILE "set xlabel offset -2\n");
	print(OUTPUTFILE "set ylabel offset 3\n");
	print(OUTPUTFILE "set ytics offset 2\n");
	print(OUTPUTFILE "set zlabel rotate by +90 left\n");
	
	print(OUTPUTFILE "set style data linespoints\n");
	
	print(OUTPUTFILE "splot '$input_file' notitle\n");
	
	print(OUTPUTFILE "set term postscript color\n");
	print(OUTPUTFILE "set term postscript eps\n");
	print(OUTPUTFILE "set output '$output_file_path/$output_file_name.eps'\n");
	print(OUTPUTFILE "set size 1, 1\n");
	print(OUTPUTFILE "replot\n");
	
	close(OUTPUTFILE);
	
	system("gnuplot <$output_file_path/template.plot");
	
	#remove template file
	if (-e "$output_file_path/template.plot"){unlink("$output_file_path/template.plot");}
}




###########################################
#
#	makeGnuplotStackedHistogram function
#
#	parameter: 
#		$input_file (with absolute path): data for gnuplot
#		$output_file_path: the directory to put the output_file
#		$output_file_name to save the graph (without extension, the extension will be .eps)
#		$title: title of the staked histogram
#		$nbparam: number of component of the stack
#
#	result:
#		the graph is save in $output_file_path/$output_file_name.eps
#
###########################################
sub makeGnuplotStackedHistogram($$$$$)
{
	my ($class,$input_file,$output_file_path,$output_file_name,$title,$nbparam)= @_;
	
	$nbparam+=1;#tweak to have the $nbparam in staked histogram
	
	#remove previous output file
	if (-e "$output_file_path/$output_file_name".".eps"){unlink("$output_file_path/$output_file_name".".eps");}
	
	#remove template file if any
	if (-e "$output_file_path/template.plot"){unlink("$output_file_path/template.plot");}
	
	open(OUTPUTFILE,">$output_file_path/template.plot") || die ("Error when opening $output_file_path/template.plot") ;
	
	print(OUTPUTFILE "set boxwidth 0.75 absolute\n");
	print(OUTPUTFILE "set style fill   solid 1.00 border lt -1\n");
	print(OUTPUTFILE "set key outside right top vertical Left reverse noenhanced autotitles columnhead nobox\n");
	print(OUTPUTFILE "set key invert samplen 4 spacing 1 width 0 height 0 \n");
	print(OUTPUTFILE "set style histogram rowstacked title  offset character 0, 0, 0\n");
	print(OUTPUTFILE "set datafile missing '-'\n");
	print(OUTPUTFILE "set style data histograms\n");
	print(OUTPUTFILE "set xtics  norangelimit font \",8\"\n");
	print(OUTPUTFILE "set xtics   ()\n");
	print(OUTPUTFILE "set title \"$title\" \n");
	print(OUTPUTFILE "i = $nbparam\n");
	print(OUTPUTFILE "plot '$input_file' using 2:xtic(1), for [i=3:$nbparam] '' using i\n");
	
	print(OUTPUTFILE "set term postscript color\n");
	print(OUTPUTFILE "set term postscript eps\n");
	print(OUTPUTFILE "set output '$output_file_path/$output_file_name.eps'\n");
	print(OUTPUTFILE "set size 1, 1\n");
	print(OUTPUTFILE "replot\n");
	
	close(OUTPUTFILE);
	
	system("gnuplot <$output_file_path/template.plot");
	
	#remove template file
	if (-e "$output_file_path/template.plot"){unlink("$output_file_path/template.plot");}
}



###########################################
#
#	makeGnuplotGroupCurve function
#
#	parameter: 
#		$input_file (with absolute path): data for gnuplot
#		$output_file_path: the directory to put the output_file
#		$output_file_name to save the graph (without extension, the extension will be .eps)
#		$title: title of the staked histogram
#		$nbparam: number of component of the stack
#
#	result:
#		the graph is save in $output_file_path/$output_file_name.eps
#
###########################################
sub makeGnuplotGroupCurve($$$$$)
{
	my ($class,$input_file,$output_file_path,$output_file_name,$title,$nbparam)= @_;
	
	$nbparam+=1;#tweak to have the $nbparam in staked histogram
	
	#remove previous output file
	if (-e "$output_file_path/$output_file_name".".eps"){unlink("$output_file_path/$output_file_name".".eps");}
	
	#remove template file if any
	if (-e "$output_file_path/template.plot"){unlink("$output_file_path/template.plot");}
	
	open(OUTPUTFILE,">$output_file_path/template.plot") || die ("Error when opening $output_file_path/template.plot") ;
	
	print(OUTPUTFILE "set key outside right top vertical Left reverse noenhanced autotitles columnhead nobox\n");
	
	print(OUTPUTFILE "set style data linespoints\n");

	print(OUTPUTFILE "set xtics border in scale 0,0 nomirror rotate by -45  offset character 0, 0, 0\n");
	print(OUTPUTFILE "set xtics  norangelimit font \",8\"\n");
	print(OUTPUTFILE "set xtics   ()\n");
	print(OUTPUTFILE "set title \"$title\" \n");
	print(OUTPUTFILE "i = $nbparam\n");
	print(OUTPUTFILE "plot '$input_file' using 2:xtic(1), for [i=3:$nbparam] '' using i\n");
	
	print(OUTPUTFILE "set term postscript color\n");
	print(OUTPUTFILE "set term postscript eps\n");
	print(OUTPUTFILE "set output '$output_file_path/$output_file_name.eps'\n");
	print(OUTPUTFILE "set size 1, 1\n");
	print(OUTPUTFILE "replot\n");
	
	close(OUTPUTFILE);
	
	system("gnuplot <$output_file_path/template.plot");
	
	#remove template file
	if (-e "$output_file_path/template.plot"){unlink("$output_file_path/template.plot");}
}


###########################################
#
#	round function
#
#	parameter: 
#		float number
#
#	result:
#		float number: x.xx format
#
###########################################
sub myround($)
{
	my ($class,$res)= @_;
	
	if($res <0){$res = int(100*$res - 0.5)/100 ;}
	else{$res = int(100*$res + 0.5)/100 ;}
	return $res;
}


####################################################
####################################################
####################################################
####################################################
1;
